#!/usr/bin/env python3

import argparse
import json
import os

from gitrics import configuration
from gitrics.bin.generate_user_project_membership import main as ProjectMembership
from gitrics.bin.parameters import param_access, param_api, param_date_end, param_date_start, param_filter_group_namespaces, param_filter_project_ids, param_filter_project_names, param_filter_project_names_starts_with, param_member_only, param_personal_only, param_simple, param_token, param_user, param_visibility
from gitrics.utilities import configure_date_range

def main(user: dict = None, date_start: str = None, date_end: str = None, projects: list = None):
    """
    Generate topics for user from GitLab usage.

    Args:
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value
        projects (list): dictionaries where each represents a GitLab Project
        user (dictionary): GitLab User
    """

    # parse arguments
    opt = parse_args()

    # convert time
    dt_start, dt_end = configure_date_range(
        date_end=date_end if date_end else opt.date_end,
        date_start=date_start if date_start else opt.date_start
    )

    # get user project membership if necessary
    if not projects:
        projects, projects_binned_by_access, count_projects_private = ProjectMembership(
            date_end=dt_end.strftime("%Y-%m-%d"),
            date_start=dt_start.strftime("%Y-%m-%d"),
            return_data=True,
            user=user
        )

    topics = dict()

    # loop through projects
    for project in projects:

        # loop through topics
        for topic in project["topics"]:

            # check if tracked already
            if topic not in topics:

                # add it
                topics[topic] = { "percent": 0 }

            # sum percents
            topics[topic]["percent"] += project["topics"][topic]

    # loop through topics
    for topic in topics:

        # sum percents
        topics[topic]["percent_aggregate"] = topics[topic]["percent"] / sum([topics[k]["percent"] for k in topics])

    # write to file
    with open("topics.json", "w") as f:
        f.write(json.dumps({
            "stats": {
                "count_projects": len(projects)
            },
            "data": {k: (topics[k]["percent_aggregate"] * 100) for k in topics}
        }))

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_access(parser)
    param_api(parser)
    param_date_end(parser)
    param_date_start(parser)
    param_filter_group_namespaces(parser)
    param_filter_project_ids(parser)
    param_filter_project_names(parser)
    param_filter_project_names_starts_with(parser)
    param_member_only(parser)
    param_personal_only(parser)
    param_simple(parser)
    param_token(parser)
    param_visibility(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import json
import os

import colorgram
import requests

from PIL import Image

from gitrics import configuration
from gitrics.bin.parameters import param_accent_color, param_api, param_token, param_user

from glapi.user.user import GitlabUser

CERT_IS_PRESENT = "REQUESTS_CLIENT_CERT" in os.environ and os.environ["REQUESTS_CLIENT_CERT"] and os.environ["REQUESTS_CLIENT_CERT"] != ""
KEY_IS_PRESENT = "REQUESTS_CLIENT_KEY" in os.environ and os.environ["REQUESTS_CLIENT_KEY"] and os.environ["REQUESTS_CLIENT_KEY"] != ""

def main(user: dict = None):
    """
    Generate user color data from GitLab avatar.

    Args:
        user (dictionary): GitLab User
    """

    # parse arguments
    opt = parse_args()

    # initialize user if necessary
    if not user:
        user = GitlabUser(
            id=opt.user,
            token=opt.token,
            version=opt.api
        ).user

    # attempt to extract color based on user avatar
    try:

        # http request for avatar
        avatar = requests.get(user["avatar_url"], cert=(os.environ["REQUESTS_CLIENT_CERT"], os.environ["REQUESTS_CLIENT_KEY"]), stream=True) if (CERT_IS_PRESENT and KEY_IS_PRESENT) else requests.get(user["avatar_url"], stream=True)

        # pull colors
        colors = colorgram.extract(Image.open(avatar.raw), 6)

    except Exception as e:
        raise e

    # generate css
    with open("visualization.scss", "w") as f:
        f.write("%s" % "\n".join([
        "$visualization%s: %s;" % (
            i + 1,
            "rgb(%s, %s, %s)" % (
                d.rgb.r,
                d.rgb.g,
                d.rgb.b
            )
        ) for i, d in enumerate(colors)
        ]))

        # if accent color provided add it last
        # to overwrite auto generated color
        if opt.accent_color:
            f.write("$visualization%s: %s;" % (
                len(colors),
                opt.accent_color
            ))

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_accent_color(parser)
    param_api(parser)
    param_token(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



import time

from gitrics import configuration

from glapi.project import GitlabProject

class gitricsProjectEvents(GitlabProject):
    """
    gitricsProjectEvents is a collection of Gitlab Project Events modified and enriched for gitrics ecosystem.
    """

    def __init__(self, project_id: str = None, project: dict = None, date_start: str = None, date_end: str = None, event_actions: list = None, token: str = None, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            date_end (string): iso 8601 date value
            date_start (string): iso 8601 date value
            project (dictionary): key/value pair representing a GitLab Project
            project_id (string): GitLab Project id
            token (string): GitLab personal access or deploy token
            version (string): GitLab API version as base url
        """

        # initialize inheritance
        super(gitricsProjectEvents, self).__init__(
            id=project_id,
            project=project,
            token=token,
            version=version
        )

        # superficial delay for api constraints
        #time.sleep(2)

        # get events
        self.events = self.extract_events(
            actions=event_actions if event_actions else list(),
            date_end=date_end,
            date_start=date_start
        )



import time

from gitrics import configuration

from glapi.user import GitlabUser

def convert_topics_to_percent(projects: list) -> list:
    """
    Convert topic list to percent format similar to default GitLab Language objects.

    Args:
        projects (list): dictionaries where each represents a GitLab Project

    Returns:
        A list of dictionaries where each represents a GitLab Project.
    """

    # loop through projects
    for project in projects:

        # capture topic object
        topics_list = project["topics"]

        # put topics in same percent format as languages
        project["topics"] = dict()

        # loop through topics
        for topic in topics_list:

            # convert counts to percent
            project["topics"][topic] = (1 / len(topics_list)) * 100

    return projects

class gitricsUserProjects(GitlabUser):
    """
    gitricsUserProjects is a collection of user-specific Gitlab Project objects modified and enriched for gitrics ecosystem.
    """

    def __init__(self, user_id: str = None, user: dict = None, access: int = configuration.USER_ACCESS, simple: bool = configuration.SIMPLE, visibility: str = configuration.VISIBILITY, membership: bool = configuration.MEMBERSHIP, personal: bool = configuration.PERSONAL, token: str = None, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            access (integer): minimum access level of a user on a given project
            membership (boolean): TRUE if api should query specific to the user ttached to the access token
            personal (boolean): TRUE if api should return namespace (personal) user projects only
            simple (boolean): TRUE if api return should be minimal
            token (string): GitLab personal access or deploy token
            user (dict): key/values representing a Gitlab User
            user_id (string): GitLab User id
            version (string): GitLab API version as base url
            visibility (enum): internal | private | public
        """

        # initialize inheritance
        super(gitricsUserProjects, self).__init__(
            id=user_id,
            token=token,
            user=user,
            version=version
        )

        # get projects
        self.projects = self.extract_projects(
            access=access,
            membership=membership,
            personal=personal,
            simple=simple,
            visibility=visibility
        )

    def bin(self, bin: str, simple: bool = configuration.SIMPLE, visibility: str = configuration.VISIBILITY, membership: bool = configuration.MEMBERSHIP, personal: bool = configuration.PERSONAL) -> dict:
        """
        Organize projects by membership access level.

        Args:
            bin (enum): access |
            membership (boolean): TRUE if api should query specific to the user ttached to the access token
            personal (boolean): TRUE if api should return namespace (personal) user projects only
            simple (boolean): TRUE if api return should be minimal
            visibility (enum): internal | private | public

        Returns:
            A dictionary where keys are the bin label and corresponding values are lists of dictionaries where each is a Gitlab project which fits the bin criteria.
        """

        result = dict()

        if bin == "access":

            # determine maximum access level
            max_level = int(max(configuration.USER_ACCESS_LEVELS.keys()) / 10)
            min_level = int(min(configuration.USER_ACCESS_LEVELS.keys()) / 10)

            result = dict()
            tracked = list()

            # determine what order to extract projects in
            # based on how gitlab exposes user access level
            # we have to loop through different queries to assemble by user access
            for digit in reversed(range(min_level, max_level + 1)):

                filtered = list()
                level = digit * 10

                # get projects user is a member of
                api_results = self.extract_projects(
                    access=level,
                    membership=membership,
                    personal=personal,
                    simple=simple,
                    visibility=visibility
                )

                # skip maximum level since all those project are valid to that access level
                if digit != max_level:

                    # remove projects of higher levels
                    filtered = [d for d in api_results if d["id"] not in tracked]

                # add ids to track
                for d in api_results:
                    if d["id"] not in tracked:
                        tracked.append(d["id"])

                # set value in map
                result[configuration.USER_ACCESS_LEVELS[level]] = api_results if digit == max_level else filtered

        return result

    def enrich(self, projects: list) -> list:
        """
        Enrich projects with foci, languages, and topics.

        Args:
            projects (list): dictionaries of GitLab Project

        Returns:
            A list of dictionaries where each represents an enriched GitLab Project.
        """

        # add languages
        projects = self.enrich_languages(projects)

        # add topics
        projects = self.enrich_topics(projects)

        # add members
        projects = self.enrich_membership(projects)

        # add foci
        projects = self.enrich_foci(projects)

        # add effort
        projects = self.enrich_effort(projects)

        # try to assign effort name if a known effort name shows up in topic list
        projects = self.extract_effort(projects)

        # clean up topics to remove effort and foci keys
        projects = self.prune_topics(projects)

        # conver topic counts to percents
        return convert_topics_to_percent(projects)

    def enrich_effort(self, projects: list) -> list:
        """
        Enrich projects with effort.

        Args:
            projects (list): dictionaries of GitLab Project

        Returns:
            A list of dictionaries where each represents an enriched GitLab Project.
        """

        result = list()

        for project in projects:

            # determine effort based on namespace
            effort = [d for d in project["path_with_namespace"].split("/")[0:-1] if d in project["name"]]

            p = project

            # add more data to new object
            p["effort"] = effort[0] if len(effort) > 0 else project["name"]

            # add to result
            result.append(p)

        return result

    def enrich_foci(self, projects: list) -> list:
        """
        Enrich projects with foci.

        Args:
            projects (list): dictionaries of GitLab Project

        Returns:
            A list of dictionaries where each represents an enriched GitLab Project.
        """

        result = list()

        for project in projects:

            p = project

            # add more data to new object
            p["foci"] = [configuration.FOCI_TERMS[d.lower()] for d in project["topics"] if d.lower() in configuration.FOCI_TERMS] if configuration.FOCI_TERMS else list()

            # add to result
            result.append(p)

        return result

    def enrich_languages(self, projects: list) -> list:
        """
        Enrich projects with languages.

        Args:
            projects (list): dictionaries of GitLab Project

        Returns:
            A list of dictionaries where each represents an enriched GitLab Project.
        """

        result = list()

        for project in projects:

            # pull languages from api
            languages = self.connection.query("projects/%s/languages" % project["id"])["data"]

            # artifical delay for api constraints
            #time.sleep(2)

            p = project

            # add more data to new object
            p["languages"] = languages

            # add to result
            result.append(p)

        return result

    def enrich_topics(self, projects: list) -> list:
        """
        Enrich projects with topics.

        Args:
            projects (list): dictionaries of GitLab Project

        Returns:
            A list of dictionaries where each represents an enriched GitLab Project.
        """

        result = list()

        for project in projects:

            # get list of languages
            languages = [d.lower() for d in project["languages"]]

            # get topics and filter out languages from topics
            topics = [d.lower() for d in project["tag_list"] if d.lower() not in languages]

            p = project

            # add more data to new object
            p["topics"] = topics

            # add to result
            result.append(p)

        return result

    def enrich_membership(self, projects: list) -> list:
        """
        Determine membership for a collection of GitLab Projects.

        Args:
            projects (list): dictionaries where each is a GitLab Project

        Returns:
            A list of dictionaries where each represents a GitLab project.
        """

        result = list()

        # loop through project ids list
        for project in projects:

            p = project

            # add more data to new object
            p["members"] = self.connection.paginate("projects/%s/members/all" % project["id"])

            # superficial delay for api constraints
            #time.sleep(2)

            # add to result
            result.append(p)

        return result

    def extract_effort(self, projects: list) -> list:
        """
        Set effort name when known effort is found in topics.

        Args:
            projects (list): dictionaries where each represent a GitLab Project

        Returns:
            A dictionary of key/value paris representing efforts.
        """

        # get effort names
        effort_names = [d["effort"].lower() for d in projects]

        # loop through projects
        for project in projects:

            # when the effort names matches the project name
            # it has been determined to be a one-off project
            # if the topic list has an existing effort name
            # this project could belong to an effort but exists in a group which isn't named for the effort
            if project["effort"] == project["name"]:

                # loop through effort names
                for effort_name in effort_names:

                    # loop through topics
                    for topic in project["topics"]:

                        # topic matches an effort
                        if topic.lower() == effort_name:

                            # update project effort
                            project["effort"] = effort_name

        return projects

    def prune(self, projects: list, filter_for_events: list = None, filter_project_ids: list = configuration.FILTER_PROJECT_IDS, filter_project_names: list = configuration.FILTER_PROJECT_NAMES, filter_project_names_startswith: list = configuration.FILTER_PROJECT_NAMES_STARTSWITH, filter_group_namespaces: list = configuration.FILTER_GROUP_NAMESPACES) -> list:
        """
        Prune projects.

        Args:
            filter_for_events (list): integers where each is a Gitlab project id with events
            filter_group_namespaces (list): strings where each is a Gitlab Group full namespace
            filter_project_ids (list): integers where each is a Gitlab Project id
            filter_project_names (list): strings where each is a Gitlab Project name
            filter_project_names_startswith (list): strings where each is a prefix of a Gitlab Project name
            projects (list): dictionaries where each is a Gitlab Project

        Returns:
            A list of dictionaries where each represents a Gitlab project.
        """

        # filter by explicit id
        result = [d for d in projects if d["id"] not in filter_project_ids]

        # filter by explicit name
        result = [d for d in result if d["name"] not in filter_project_names]

        # loop through project name prefixes
        for prefix in filter_project_names_startswith:

            # filter by name starts with
            result = [d for d in result if not d["name"].startswith(prefix)]

        # loop through group namespaces
        for namespace in filter_group_namespaces:

            # loop through group name spaces
            result = [d for d in result if namespace not in  d["path_with_namespace"]]

        # filter for projects with events
        if filter_for_events:
            result = [d for d in result if d["id"] in filter_for_events]

        return result

    def prune_topics(self, projects: list) -> list:
        """
        Remove efforts and foci from topics.

        Args:
            projects (list): dictionaries where each represents a GitLab Project

        Returns:
            A list of dictionaries where each represent a GitLab Project.
        """

        # get effort names
        effort_names = [d["effort"].lower() for d in projects]

        # loop through projects
        for project in projects:

            # loop through topics
            for topic in project["topics"]:

                # remove effort name from topics
                project["topics"] = [d for d in project["topics"] if d.lower() not in effort_names]

                # remove foci from topics
                project["topics"] = [d for d in project["topics"] if d.lower() not in list(configuration.FOCI_TERMS.keys())] if configuration.FOCI_TERMS else list()

        return projects



import datetime
import math

import scipy.stats as st

def ci_lower_bound(ratings: int, total: int, confidence: float = 0.95) -> float:
    """
    Calculate the lower bound on the proportion of the rating, i.e. balance the proportion of ratings with the uncertainty of a small number of observations.

    Reference: https://www.evanmiller.org/how-not-to-sort-by-average-rating.html

    Args:
        confidence (float): statistical confidence level that the lower bound is correct
        ratings (integer): count of ratings to score by
        total (integer): total count of all ratings

    Returns:
        A float representing the score for of an item relative to all items where all items have the potential to be rated via an ordinal scale toward an action.
    """

    result = 0

    # check for totals
    if total > 0:

        # get standard deviation multipler for percentage value
        z = st.norm.ppf(1 - (1 - confidence) / 2)
        phat = 1.0 * ratings / total

        # update result
        result = (phat + z * z / (2 * total) - z * math.sqrt((phat * (1 - phat) + z * z / (4 * total)) / total)) / (1 + z * z / total)

    return result

def configure_date_range(date_start, date_end):
    """
    Determine start/end dates based on user input.

    Args:
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value

    Returns:
        A tuple of start, end dateimte values.
    """

    # convert time
    dt_end = datetime.datetime.strptime(date_end, "%Y-%m-%d")
    dt_start = datetime.datetime.strptime(date_start, "%Y-%m-%d") if date_start and date_end and date_start != date_end else dt_end - datetime.timedelta(3)

    return (dt_start, dt_end)

def parse_multi_parameters(value):
    """
    Parse a string of comma-delimetered values into a native python list.

    Args:
        value (string): comma-delimetered value

    Returns:
        A list of strings where each is a discrete value in the set.
    """

    return value.split(",") if value else list()



#!/usr/bin/env python3

import argparse
import json
import os
import time

from gitrics import configuration
from gitrics.bin.generate_dependency_metadata import main as Dependency
from gitrics.bin.generate_metadata import main as Metadata
from gitrics.bin.generate_user_activity import main as Activity
from gitrics.bin.generate_user_color import main as Color
from gitrics.bin.generate_user_contribution import main as Contribution
from gitrics.bin.generate_user_connection import main as Connection
from gitrics.bin.generate_user_data import main as UserMain
from gitrics.bin.generate_user_data_coverage import main as Coverage
from gitrics.bin.generate_user_languages import main as Languages
from gitrics.bin.generate_user_project_membership import main as ProjectMembership
from gitrics.bin.generate_user_topics import main as Topics
from gitrics.bin.parameters import param_access, param_api, param_date_end, param_date_start, param_filter_group_namespaces, param_filter_project_ids, param_filter_project_names, param_filter_project_names_starts_with, param_member_only, param_personal_only, param_simple, param_token, param_user, param_visibility
from gitrics.utilities import configure_date_range

def main():
    """
    Generate complete data profile for user from GitLab usage.
    """

    # parse arguments
    opt = parse_args()

    # convert time
    dt_start, dt_end = configure_date_range(opt.date_start, opt.date_end)

    # generate metadata
    Dependency()
    Metadata(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d")
    )

    # generate user data
    user = UserMain(return_data=True)

    # rate imit on api
    time.sleep(10)

    # generate projects data
    projects, projects_binned_by_access, count_projects_private = ProjectMembership(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        return_data=True,
        user=user
    )

    # rate imit on api
    time.sleep(5)

    # generate color palette
    Color(user=user)

    # generate user connections
    Connection(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        projects=projects,
        user=user
    )

    # generate data coverage
    Coverage(
        count_projects_private=count_projects_private,
        projects=projects,
        projects_binned_by_access=projects_binned_by_access,
        user=user
    )

    # generate languages
    Languages(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        projects=projects,
        user=user
    )

    # generate topics
    Topics(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        projects=projects,
        user=user
    )

    # generate events
    Activity(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        projects=projects,
        user=user
    )

    # generate effort contribution
    Contribution(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        projects=projects,
        user=user
    )
    
def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_access(parser)
    param_api(parser)
    param_date_end(parser)
    param_date_start(parser)
    param_filter_group_namespaces(parser)
    param_filter_project_ids(parser)
    param_filter_project_names(parser)
    param_filter_project_names_starts_with(parser)
    param_member_only(parser)
    param_personal_only(parser)
    param_simple(parser)
    param_token(parser)
    param_visibility(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import json
import os

from collections import Counter

from gitrics import configuration
from gitrics.bin.parameters import param_user, param_access, param_api, param_filter_group_namespaces, param_filter_project_ids, param_filter_project_names, param_filter_project_names_starts_with, param_member_only, param_personal_only, param_return_data, param_simple, param_token, param_visibility
from gitrics.user.projects import gitricsUserProjects
from gitrics.utilities import parse_multi_parameters

def main(user: dict = None, date_start: str = None, date_end: str = None, return_data: bool = False):
    """
    Generate complete data profile for user from GitLab usage.

    Args:
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value
        return_data (bool): TRUE if data should be returned
        user (dictionary): GitLab User

    Return:
        A tuple of (list, dict, int) where list of dictionaries where each represents a GitLab Project, dict is a key/pair value sets of project visibility to projects, and int is the count of private projects.
    """

    # parse arguments
    opt = parse_args()

    # initialize projects
    user_projects = gitricsUserProjects(
        access=opt.access,
        membership=opt.member_only,
        personal=opt.personal_only,
        simple=opt.simple,
        user=user,
        user_id=opt.user,
        token=opt.token,
        version=opt.api,
        visibility=opt.visibility
    )

    # depending on how user data is passed to script or
    # as function argument we can still pull the user object
    # from the initialized class without needing to requery the API
    user = user_projects.user

    # prune projects based on specified items
    projects = user_projects.prune(
        user_projects.projects,
        filter_group_namespaces=parse_multi_parameters(opt.filter_group_namespaces),
        filter_project_ids=parse_multi_parameters(opt.filter_project_ids),
        filter_project_names=parse_multi_parameters(opt.filter_project_names),
        filter_project_names_startswith=parse_multi_parameters(opt.filter_project_names_startswith)
    )

    # bin projects by user access level
    projects_binned_by_access = user_projects.bin("access")

    # enrich projects with langauges/topics/foci/effort/membership
    projects = user_projects.enrich(projects)

    # find out how many private projects the user has
    # regardless of namespace or public
    count_projects_private = len(user_projects.connection.paginate("projects", { "visibility": "private"}))

    # calculate simple project totals
    total_projects = len(user_projects.projects)
    total_projects_developer = len(projects_binned_by_access["developer"])
    total_projects_maintainer = len([d for d in projects_binned_by_access["maintainer"] if user["username"] not in d["path_with_namespace"]])
    total_projects_personal = len([d for d in projects_binned_by_access["maintainer"] if user["username"] in d["path_with_namespace"]])
    total_projects_public = total_projects - total_projects_personal
    total_projects_owner = len(projects_binned_by_access["owner"])

    # calculate public to personal
    public_personal_difference = total_projects_public -  total_projects_personal;
    public_personal_average = total_projects / 2;
    ratio_of_public_personal_difference_average = public_personal_difference / public_personal_average;
    public_private_percent_difference = ratio_of_public_personal_difference_average * 100

    # generate dictionary of role values
    d = {
        "developer": total_projects_developer,
        "maintainer": total_projects_maintainer,
        "owner": total_projects_owner
    }

    # find largest role percent
    count_role_percent = Counter(d)
    greatest_percent_role = max(count_role_percent, key=count_role_percent.get)

    # add project types to roles
    d["personal"] = total_projects_personal
    d["private"] = count_projects_private
    d["public"] = total_projects_public

    # write stats to file
    with open("membership.json", "w") as f:
        f.write(json.dumps({
            "stats": {
                "difference_public_to_personal": public_personal_difference,
                "greatest_percent_role": greatest_percent_role,
                "percent_developer": int(round(total_projects_developer / total_projects_public) * 100) if total_projects_public > 0 else None,
                "percent_difference_public_to_personal": int(round(public_private_percent_difference)),
                "percent_maintainer": int(round(total_projects_maintainer / total_projects_public) * 100) if total_projects_public > 0 else None,
                "percent_owner": int(round(total_projects_owner / total_projects_public) * 100) if total_projects_public > 0 else None,
                "percent_personal": int(round((total_projects_personal / total_projects) * 100)) if total_projects_public > 0 else None,
                "percent_public": int(round((total_projects_public / total_projects) * 100)) if total_projects > 0 else None,
                "count_projects": total_projects
            },
            "data": d
        }))

    # only return data when requested 
    # this ensures console scripts do not break when output is evaluated as error 1 code
    if opt.return_data or return_data:
        return (projects, projects_binned_by_access, count_projects_private)

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_access(parser)
    param_api(parser)
    param_filter_group_namespaces(parser)
    param_filter_project_ids(parser)
    param_filter_project_names(parser)
    param_filter_project_names_starts_with(parser)
    param_member_only(parser)
    param_personal_only(parser)
    param_return_data(parser)
    param_simple(parser)
    param_token(parser)
    param_visibility(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import json
import os

from gitrics import configuration
from gitrics.bin.generate_user_project_membership import main as ProjectMembership
from gitrics.bin.parameters import param_api, param_date_end, param_date_start, param_token, param_user
from gitrics.user.issues import gitricsUserIssues
from gitrics.user.merge_requests import gitricsUserMergeRequests
from gitrics.utilities import configure_date_range

def main(user: dict = None, date_start: str = None, date_end: str = None, projects: list = None):
    """
    Generate complete data profile for user from GitLab usage.

    Args:
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value
        projects (list): dictionaries where each represents a GitLab Project
        user (dictionary): GitLab User
    """

    # parse arguments
    opt = parse_args()

    # convert time
    dt_start, dt_end = configure_date_range(
        date_end=date_end if date_end else opt.date_end,
        date_start=date_start if date_start else opt.date_start
    )

    # initialie issues
    user_issues = gitricsUserIssues(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        token=opt.token,
        user_id=opt.user,
        version=opt.api
    )

    # pull user data from class
    user = user if user else user_issues.user

    # prune to remove created by and assigned to same user
    issues = user_issues.prune(user_issues.items)

    # format as key/value map of user id to related user object
    issues_formatted = user_issues.format(issues)

    # initialie merge requests
    user_merge_requests = gitricsUserMergeRequests(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        token=opt.token,
        user_id=opt.user,
        version=opt.api
    )

    # prune to remove created by and assigned to same user
    merge_requests = user_merge_requests.prune(user_merge_requests.items)

    # format as key/value map of user id to related user object
    merge_request_formatted = user_merge_requests.format(merge_requests)

    # combine totals for issues/merge requests
    count_issues_mergerequests = dict()

    # loop through user ids from issues
    for user_id in issues_formatted:

        # check for key in combined
        if user_id not in count_issues_mergerequests:

            # add key/update attributes
            count_issues_mergerequests[user_id] = issues_formatted[user_id]
            count_issues_mergerequests[user_id]["member_only"] = False

    # loop through user ids from merge requests
    for user_id in merge_request_formatted:

        # check for key in combined
        if user_id not in count_issues_mergerequests:

            # add key/update attributes
            count_issues_mergerequests[user_id] = { "count": 0, "member_only": False }

        # now that key exists sum values not yet captured
        count_issues_mergerequests[user_id]["count"] += merge_request_formatted[user_id]["count"]

    # get user project membership if necessary
    if not projects:
        projects, projects_binned_by_access, count_projects_private = ProjectMembership(
            date_end=dt_end.strftime("%Y-%m-%d"),
            date_start=dt_start.strftime("%Y-%m-%d"),
            return_data=True,
            user=user
        )

    # extract and format active project memebers
    # filtering out users which match self
    level1_connections = [
        {
            "id": d["id"],
            "members": [
                {
                    "id": m["id"],
                    "name": m["name"],
                    "avatar_url": m["avatar_url"],
                    "web_url": m["web_url"]
                } for m in d["members"]
                if m["state"] == "active" and m["id"] != user["id"]
            ]
        } for d in projects
    ]

    # flatten
    level1_connections_flat = [x for y in [d["members"] for d in level1_connections] for x in y]

    # remove duplicates
    level1_connections = [dict(t) for t in {tuple(d.items()) for d in level1_connections_flat}]

    # loop through first level connections
    for user in level1_connections:

        # check if captured in combined connection counts
        if user["id"] not in count_issues_mergerequests:

            # add user and update attribute for membership
            count_issues_mergerequests[user["id"]] = {
                "user": user,
                "count": 0,
                "member_only": True
            }

            # add value to existing
            count_issues_mergerequests[user["id"]]["count"] += 1

    # write languages to file
    with open("connections.json", "w") as f:
        f.write(json.dumps({
            "stats": dict(),
            "data": count_issues_mergerequests
        }))

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_api(parser)
    param_date_end(parser)
    param_date_start(parser)
    param_token(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import json
import os

from collections import Counter

from gitrics import configuration
from gitrics.bin.generate_user_data import main as UserMain
from gitrics.bin.generate_user_project_membership import main as ProjectMembership
from gitrics.bin.parameters import param_api, param_date_end, param_date_start, param_token, param_user
from gitrics.project.events import gitricsProjectEvents
from gitrics.utilities import configure_date_range

def enrich_projects(projects: list, date_start: str, date_end: str, token: str, api: str) -> dict:
    """
    Add events and membership which are filtered by each other and added to each project as attributes.

    Args:
        api (string): GitLab api url
        date_end (string): iso 8601 date value
        date_start (string): iso 8501 date value
        projects (list): dictionaries where each is a GitLab Project
        token (string): GitLab access token

    Returns:
        A list of dictionaries where each represents an enriched GitLab Project.
    """

    result = list()

    # loop through projects
    for project in projects:

        # only get events for projects with a focus tag
        if len(project["foci"]) > 0:

            # get events
            events = gitricsProjectEvents(
                date_end=date_end,
                date_start=date_start,
                project=project,
                token=token,
                version=api
            ).events

            # simplify for just the attributes needed
            events = [
                {
                    "id": d["id"],
                    "project_id": d["project_id"],
                    "author_id": d["author_id"],
                    "created_at": d["created_at"]
                } for d in events
            ]

        else:
            events = list()

        # filter by membership this way we remove events from service accounts
        events = [d for d in events if d["author_id"] in [d["id"] for d in project["members"]]]

        # ids
        member_ids_with_events = list(set([
            d["author_id"] for d in events
        ]))
        member_ids_by_focus = {
            f: [d["id"] for d in project["members"]]
            for f in project["foci"]
        }

        # counts
        foci_event_counts = {
            f: len(events) / len(project["foci"])
            for f in project["foci"]
        }

        result.append({
            "effort": project["effort"],
            "events": events,
            "foci": project["foci"],
            "foci_event_counts": foci_event_counts,
            "forks_count": project["forks_count"],
            "id": project["id"],
            "languages": project["languages"],
            "members": project["members"],
            "member_ids_by_focus": member_ids_by_focus,
            "member_ids_with_events": member_ids_with_events,
            "name": project["name"],
            "star_count": project["star_count"],
            "topics": project["topics"],
            "web_url": project["web_url"]
        })

    return result

def extract_efforts(projects: list) -> dict:
    """
    Organize projects into efforts and aggregate/organize data across projects.

    Args:
        projects (list): dictionaries where each is a GitLab Project

    Returns:
        A dictionary of key/value pairs where the key is an effort label and corresponding value is a dictionary of values representing the effort.
    """

    # the x for y in [] for x in y sytnax is flattening a 2D array into 1D
    result = {
        k: {
            "events": [
                x for y in [
                    p["events"] for p in projects if p["effort"] == k
                ] for x in y
            ],
            "foci": list(set([
                x for y in [
                    p["foci"] for p in projects if p["effort"] == k
                ] for x in y
            ])),
            "foci_event_counts": {
                f: sum([
                    p["foci_event_counts"][f] for p in projects
                        if p["effort"] == k
                        and len(p["foci"]) > 0
                        and f in p["foci"]
                ])
                for f in list(set([
                    x for y in [
                        p["foci"] for p in projects if p["effort"] == k and len(p["events"]) > 0
                    ] for x in y
                ]))
            },
            "foci_with_events": list(set([
                x for y in [
                    p["foci"] for p in projects if p["effort"] == k and len(p["events"]) > 0
                ] for x in y
            ])),
            "members": [
                dict(t) for t in
                {
                    tuple(d.items()) for d in [
                        {
                            "avatar_url": d["avatar_url"],
                            "id": d["id"],
                            "name": d["name"]
                        } for d in [
                            x for y in [
                                p["members"] for p in projects if p["effort"] == k
                            ] for x in y
                        ]
                    ]
                }
            ],
            "member_ids": list(set([
                x for y in [
                    [
                        m["id"] for m in p["members"]
                    ] for p in projects if p["effort"] == k
                ] for x in y
            ])),
            "member_ids_by_focus": {
                f: list(set([
                    x for y in [
                        p["member_ids_by_focus"][f] for p in projects
                            if p["effort"] == k
                            and len(p["member_ids_by_focus"]) > 0
                            and f in p["member_ids_by_focus"]
                    ] for x in y
                ]))
                for f in list(set([
                    x for y in [
                        p["foci"] for p in projects if p["effort"] == k
                    ] for x in y
                ]))
            },
            "member_ids_with_events": list(set([
                x for y in [
                    p["member_ids_with_events"] for p in projects if p["effort"] == k
                ] for x in y
            ])),
            "member_ids_with_foci": list(set([
                x["id"] for y in [
                    p["members"] for p in projects if p["effort"] == k and len(p["foci"]) > 0
                ] for x in y
            ])),
            "project_ids": list(set([
                p["id"] for p in projects if p["effort"] == k
            ])),
            "project_ids_with_events": list(set([
                x["project_id"] for y in [
                    p["events"] for p in projects if p["effort"] == k
                ] for x in y
            ])),
            "project_ids_by_foci": {
                f: [
                    p["id"] for p in projects if p["effort"] == k and f in p["foci"]
                ]
                for f in list(set([
                    x for y in [
                        p["foci"] for p in projects if p["effort"] == k
                    ] for x in y
                ]))
            },
            "project_ids_with_foci": list(set([
                p["id"] for p in projects if p["effort"] == k and len(p["foci"]) > 0
            ]))
        }
        for k in list(set([d["effort"] for d in projects]))
    }

    # only keep efforts with events and foci
    result = { k: result[k] for k in result if len(result[k]["events"]) > 0 and len(result[k]["foci"]) > 0 }

    return result

def main(user: dict = None, date_start: str = None, date_end: str = None, projects: list = None):
    """
    Generate effort contribution for user from GitLab usage.

    Args:
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value
        projects (list): dictionaries where each represents a GitLab Project
        user (dictionary): GitLab User
    """

    # parse arguments
    opt = parse_args()

    # convert time
    dt_start, dt_end = configure_date_range(
        date_end=date_end if date_end else opt.date_end,
        date_start=date_start if date_start else opt.date_start
    )

    # dates
    date_start = dt_start.strftime("%Y-%m-%d")
    date_end = dt_end.strftime("%Y-%m-%d")

    # generate user data if necessary
    if not user:
        user = UserMain()

    # get user project membership if necessary
    if not projects:
        projects, projects_binned_by_access, count_projects_private = ProjectMembership(
            date_end=date_end,
            date_start=date_start,
            return_data=True,
            user=user
        )

    # enrich projects with events/filter by membership
    projects = enrich_projects(projects, date_start, date_end, opt.token, opt.api)

    # organize projects into efforts
    efforts = extract_efforts(projects)

    # get across all efforts
    all_events = [
        x for y in [
            efforts[k]["events"] for k in efforts
        ] for x in y
    ]
    all_foci = list(set([
        x for y in [
            efforts[k]["foci"] for k in efforts
        ] for x in y
    ]))
    all_project_ids_with_events = list(set([
        x for y in [
            efforts[k]["project_ids_with_events"] for k in efforts
        ] for x in y
    ]))
    all_member_ids_with_events = list(set([
        x for y in [
            efforts[k]["member_ids_with_events"] for k in efforts
        ] for x in y
    ]))

    # generate reference objects across all efforts
    connections = {
        f: {
            e: [
                m for m in efforts[e]["member_ids_with_events"]
                if m in efforts[e]["member_ids_with_foci"]
            ]
            for e in efforts
            if f in efforts[e]["foci"]
        }
        for f in all_foci
    }
    foci_reference_project_ids = {
        f: list(set([
            x for y in [
                efforts[k]["project_ids_by_foci"][f] for k in efforts if f in efforts[k]["project_ids_by_foci"]
            ] for x in y
        ]))
        for f in all_foci
    }
    member_ids_with_events_by_focus = {
        f: list(set([
            x for y in [
                [
                    u for u in efforts[k]["member_ids_by_focus"][f]
                        if u in efforts[k]["member_ids_with_events"]
                ] for k in efforts
                    if f in efforts[k]["member_ids_by_focus"]
            ] for x in y
        ]))
        for f in all_foci
    }
    member_reference = {
        u: [
            x for y in [
                efforts[k]["members"] for k in efforts
            ] for x in y if x["id"] == u
        ][0]
        for u in all_member_ids_with_events
    }
    project_ids_by_effort = {
        k: efforts[k]["project_ids"]
        for k in efforts
    }
    project_ids_with_foci_and_events = list(set([
        x for y in [
            [
                p for p in efforts[k]["project_ids_with_events"]
                    if p in efforts[k]["project_ids_with_foci"]
            ]
            for k in efforts
        ] for x in y
    ]))
    project_ids_with_foci_and_events_by_effort = {
        k: [
            p for p in efforts[k]["project_ids_with_events"]
                if p in efforts[k]["project_ids_with_foci"]
        ]
        for k in efforts
    }

    # count events
    effort_event_counts = {
        k: len([
            e for e in efforts[k]["events"]
        ])
        for k in efforts
    }
    effort_event_counts_with_foci = {
        k: len([
            e for e in efforts[k]["events"]
                if e["author_id"] in efforts[k]["member_ids_with_foci"]
                and e["project_id"] in efforts[k]["project_ids_with_foci"]
        ])
        for k in efforts
    }
    foci_event_counts = {
        f: sum([
            efforts[k]["foci_event_counts"][f]
            for k in efforts
                if f in efforts[k]["foci_event_counts"]
        ])
        for f in all_foci
    }
    member_event_counts_with_foci = {
        m: sum([
            len([
                    e for e in efforts[k]["events"]
                        if e["author_id"] == m
                        and e["project_id"] in efforts[k]["project_ids_with_foci"]
                ]) for k in efforts
                    if m in efforts[k]["member_ids_with_foci"]
        ])
        for m in all_member_ids_with_events
    }
    member_project_counts_with_foci_and_events = {
        m: sum([
            len(list(set([
                    e["project_id"] for e in efforts[k]["events"]
                        if e["author_id"] == m
                        and e["project_id"] in all_project_ids_with_events
                ]) for k in efforts
                    if m in efforts[k]["member_ids_with_foci"]
                    and m in efforts[k]["member_ids_with_events"]
        ))])
        for m in all_member_ids_with_events
    }
    total_events_with_foci = sum([foci_event_counts[k] for k in foci_event_counts])

    # get events specific to user
    user_events = [
        d["id"] for d in all_events
            if d["author_id"] == user["id"]
            and d["project_id"] in project_ids_with_foci_and_events
    ]

    # stats

    # averages
    average_activity_per_effort = int(round(total_events_with_foci / len(effort_event_counts_with_foci))) if len(effort_event_counts_with_foci) > 0 else None
    average_activity_per_focus = int(round(total_events_with_foci / len(all_foci))) if len(all_foci) > 0 else None

    # extract the project id of the most events
    max_event_count = Counter(user_events)
    max_event_project_id = max_event_count.most_common(1)[0][0] if len(max_event_count) > 0 else None
    max_event_project_count = max_event_count.most_common(1)[0][1] if len(max_event_count) > 0 else None

    # calculate percentile rank of activity count
    user_occurance_counts = Counter([
        d["author_id"] for d in all_events
    ])
    values_below_user_count = len([
        k for k in user_occurance_counts
        if user_occurance_counts[k] < user_occurance_counts[user["id"]
    ]])
    percentile = (values_below_user_count / len(user_occurance_counts)) * 100 if len(user_occurance_counts) > 0 else None
    percentile_rank = percentile / (100 * (len(user_occurance_counts) + 1)) if percentile else None

    # generate ordinal number format
    ordinal = lambda n: "%d%s" % (n,"tsnrhtdd"[(n//10%10!=1)*(n%10<4)*n%10::4])

    percentile_for_activity_contribution = ordinal(int(round(percentile))) if percentile else None

    # write to file
    with open("contribution.json", "w") as f:
        f.write(json.dumps({
            "stats": {
                "average_activity_per_effort": average_activity_per_effort,
                "average_activity_per_focus": average_activity_per_focus,
                "member_event_counts_with_foci": member_event_counts_with_foci,
                "member_project_counts_with_foci_and_events": member_project_counts_with_foci_and_events,
                "percentile_for_activity_contribution": percentile_for_activity_contribution
            },
            "data": {
                "efforts": effort_event_counts_with_foci,
                "foci": foci_event_counts,
                "members": member_event_counts_with_foci
            },
            "reference": {
                "connections": connections,
                "member_ids_with_events_by_focus": member_ids_with_events_by_focus,
                "members": member_reference,
                "project_ids_by_effort": project_ids_by_effort,
                "project_ids_with_foci_and_events_by_effort": project_ids_with_foci_and_events_by_effort
            }
        })
    )

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_api(parser)
    param_date_end(parser)
    param_date_start(parser)
    param_token(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



import datetime
import json
import os

from glapi import configuration as glapi_config

# FILE SYSTEM
FILEPATH_GITRICS_USER = os.environ["FILEPATH_GITRICS_USER"] if "FILEPATH_GITRICS_USER" in os.environ else None

# FOCI
FOCI = [d.strip() for d in os.environ["FOCI"].split(",")] if "FOCI" in os.environ else ["a", "b", "c", "d"]
FOCI_TERMS = None
if os.path.exists(os.path.join(os.getcwd(), "foci.json")):
    with open("foci.json", "r") as f:
        term_map = json.loads(f.read())
        FOCI_TERMS = dict()
        for key in term_map:
            for value in term_map[key]:
                FOCI_TERMS[value] = key

# GITLAB
# events
ACTIVITY_TYPES = {
    "coding": ["pushed", "merged"],
    "collaboration": ["approved", "closed", "commented", "created", "updated"]
}

# filters
FILTER_GROUP_NAMESPACES = [d.strip() for d in os.environ["FILTER_GITLAB_GROUP_NAMESPACES"].split(",")] if "FILTER_GITLAB_GROUP_NAMESPACES" in os.environ else list()
FILTER_PROJECT_IDS = [d.strip() for d in os.environ["FILTER_GITLAB_PROJECT_IDS"].split(",")] if "FILTER_GITLAB_PROJECT_IDS" in os.environ else list()
FILTER_PROJECT_NAMES = [d.strip() for d in os.environ["FILTER_GITLAB_PROJECT_NAMES"].split(",")] if "FILTER_GITLAB_PROJECT_NAMES" in os.environ else list()
FILTER_PROJECT_NAMES_STARTSWITH = [d.strip() for d in os.environ["FILTER_GITLAB_PROJECT_NAMES_STARTSWITH"].split(",")] if "FILTER_GITLAB_PROJECT_NAMES_STARTSWITH" in os.environ else list()

# instance
GITLAB_TOKEN = glapi_config.GITLAB_TOKEN
GITLAB_API_VERSION = glapi_config.GITLAB_API_VERSION
GITLAB_NAME = os.environ["GITLAB_NAME"] if "GITLAB_NAME" in os.environ else "GitLab"
GITLAB_NAMESPACE = os.environ["GITLAB_NAMESPACE"] if "GITLAB_NAMESPACE" in os.environ else None
GITLAB_URL = os.environ["GITLAB_URL"] if "GITLAB_URL" in os.environ else "https://gitlab.com"

# projects
MEMBERSHIP = glapi_config.GITLAB_PROJECT_USER_MEMBERSHIP
PERSONAL = glapi_config.GITLAB_PROJECT_PERSONAL_ONLY
SIMPLE = glapi_config.GITLAB_PROJECT_SIMPLE
VISIBILITY = glapi_config.GITLAB_PROJECT_VISIBILITY

# users
USER = os.environ["GITLAB_USER_ID"] if "GITLAB_USER_ID" in os.environ else None
USER_ACCESS = os.environ["GITLAB_USER_PROJECT_ACCESS"] if "GITLAB_USER_PROJECT_ACCESS" in os.environ else 30
USER_ACCESS_LEVELS = {
    30: "developer",
    40: "maintainer",
    50: "owner"
}

# TIME
DATE_ISO_8601 = glapi_config.DATE_ISO_8601



from gitrics import configuration

from glapi.user import GitlabUser

class gitricsUserEvents(GitlabUser):
    """
    gitricsUserEvents is a collection of user-specific Gitlab Event objects modified and enriched for gitrics ecosystem.
    """

    def __init__(self, user_id: str = None, user: dict = None, date_start: str = None, date_end: str = None, token: str = None, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            date_end (string): iso 8601 date value
            date_start (string): iso 8601 date value
            token (string): GitLab personal access or deploy token
            user (dict): key/values representing a Gitlab User
            user_id (string): GitLab User id
            version (string): GitLab API version as base url
        """

        # initialize inheritance
        super(gitricsUserEvents, self).__init__(
            id=user_id,
            token=token,
            user=user,
            version=version
        )

        # get action types from config
        action_types = [item for sublist in [[x for x in configuration.ACTIVITY_TYPES[d]] for d in configuration.ACTIVITY_TYPES] for item in sublist]

        # get events
        self.events = self.extract_events(
            actions=action_types,
            date_end=date_end,
            date_start=date_start
        )

    def prune(self, events: list, project_ids: list = list()) -> list:
        """
        Prune events.

        Args:
            events (list): dictionaries where each is a Gitlab Event
            project_ids (list): integers where each is a Gitlab Project id

        Returns:
            A list of dictionaries where each represents a Gitlab Event object.
        """

        result = list()

        # filter for specific projects
        result = [d for d in events if d["project_id"] in project_ids]

        return result



#!/usr/bin/env python3

import argparse

import networkx as nx

from gitrics import configuration
from gitrics.group.issues import gitricsGroupIssues
from gitrics.bin.parameters import param_api, param_group, param_token

def main(group_id: str = None, issues: list = None) -> list:
    """
    Generate list of blockers in directional network of blocking issues.

    Args:
        group_id (string): GitLab Group id
        issues (list): dictionaries where each is a GitLab Issue

    Returns:
        A list of x.
    """

    result = None

    # parse arguments
    opt = parse_args()

    # get issues in group
    ggi = gitricsGroupIssues(
        group_id=group_id if group_id else opt.group,
        issues=issues,
        token=opt.token,
        version=opt.api
    )

    # check for issues
    if ggi.nested:

        # calculate degree centrality
        dc = nx.degree_centrality(ggi.nested)

        # remove root node
        dc.pop(0)

        # get max value
        max_block_centrality = max(dc, key=dc.get)

        # get any nodes with max value
        result = sorted([
            d for d in ggi.issues
            if d.id in dc and
            dc[d.id] == dc[max_block_centrality]
        ], key=lambda d: nx.descendants(ggi.nested, d.id), reverse=True)

    return result

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define optional values
    param_api(parser)
    param_group(parser)
    param_token(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import json
import os

from gitrics import configuration
from gitrics.bin.parameters import param_api, param_return_data, param_token, param_user

from glapi.user.user import GitlabUser

def main(return_data: bool = False) -> dict:
    """
    Generate user data from GitLab usage.

    Args:
        return_data (bool): TRUE if data should be returned

    Returns:
        A dictionary representing a GitLab User.
    """

    # parse arguments
    opt = parse_args()

    # initialize user
    gu = GitlabUser(
        id=opt.user,
        token=opt.token,
        version=opt.api
    )

    # write user to file
    with open("user.json", "w") as f:
        f.write(json.dumps({
            "id": gu.user["id"],
            "name": gu.user["name"],
            "avatar_url": gu.user["avatar_url"],
            "web_url": gu.user["web_url"],
            "email": gu.user["public_email"]
        }))

    # only return data when requested
    # this ensures console scripts do not break when output is evaluated as error 1 code
    if opt.return_data or return_data:
        return gu.user

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_api(parser)
    param_return_data(parser)
    param_token(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import datetime
import json

from gitrics import configuration
from gitrics.bin.parameters import param_date_end, param_date_start, param_gitlab_name, param_gitlab_url, param_visibility
from gitrics.utilities import configure_date_range

def main(date_start: str = None, date_end: str = None):
    """
    Generate metadata for report.

    Args:
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value
    """

    # parse arguments
    opt = parse_args()

    # convert time
    dt_start, dt_end = configure_date_range(
        date_end=date_end if date_end else opt.date_end,
        date_start=date_start if date_start else opt.date_start
    )

    # write metadata to file
    with open("metadata.json", "w") as f:
        f.write(json.dumps({
            "date_end": dt_end.strftime("%Y-%m-%d"),
            "date_publish": datetime.datetime.now().isoformat(),
            "date_start": dt_start.strftime("%Y-%m-%d"),
            "gitlab_name": opt.gitlab_name,
            "gitlab_url": opt.gitlab_url,
            "public_or_private": "public" if opt.visibility.lower() == "internal" or opt.visibility.lower() == "public" else "private",
            "visibility": opt.visibility
        }))

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define optional values
    param_date_end(parser)
    param_date_start(parser)
    param_gitlab_name(parser)
    param_gitlab_url(parser)
    param_visibility(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



import datetime

from collections import Counter

import networkx as nx

from glapi.group import GitlabGroup
from glapi.epic import GitlabEpic
from glapi.user import GitlabUser

from gitrics import configuration
from gitrics.epic import gitricsEpic
from gitrics.issue import gitricsIssue

class gitricsItemSet:
    def __init__(self, type: str):
        self.items = None
        self.graph = None
        self.gitlab_type = type

class gitricsGroup(GitlabGroup):
    """
    gitricsGroup is an abstraction of the GitLab Group data object specific to the requirements of the gitrics ecosystem.
    """

    def __init__(self, id: str = None, group: dict = None, epics: gitricsItemSet = None, issues: gitricsItemSet = None, users: list = None, date_start: str = None, date_end: str = None, token: str = configuration.GITLAB_TOKEN, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            date_end (string): iso 8601 date value
            date_start (string): iso 8601 date value
            group (dict): key/values representing a Gitlab Group
            id (string): GitLab Group id
            epics (gitricsItemSet): classes of gitricsEpic
            issues (gitricsItemSet): classes of gitricsIssue
            token (string): GitLab personal access or deploy token
            users (list): dictionaries of GitLab User
            version (string): GitLab API version as base url
        """

        self.date_end = date_end
        self.date_start = date_start

        # initialize inheritance
        super(gitricsGroup, self).__init__(
            group=group,
            id=id,
            token=token,
            version=version
        )

        self.epics = epics if epics else gitricsItemSet("epics")
        self.issues = issues if issues else gitricsItemSet("issues")

        # get users
        self.users = self.extract_users()

    def determine_most_at_risk(self, items: list) -> list:
        """
        Determine which Epics are most at risk based on health status at_risk attributes.

        Args:
            items (list): classes of gitricsEpic

        Returns:
            A list of giticsEpic objects where each represents a GitLab Epic.
        """

        # generate maps of ids to scores
        atriskmap = { k.id: k.score_at_risk for k in items }

        # determine scoring ranks
        return [d for d in items if d.score_at_risk == atriskmap[max(atriskmap, key=atriskmap.get)]]

    def determine_most_on_track(self, items: list) -> list:
        """
        Determine which Epics are most on track based on health status on_track attributes.

        Args:
            items (list): classes of gitricsEpic

        Returns:
            A list of giticsEpic objects where each represents a GitLab Epic.
        """

        # generate maps of ids to scores
        ontrackmap = { k.id: k.score_on_track for k in items }

        # determine scoring ranks
        return [d for d in items if d.score_on_track == ontrackmap[max(ontrackmap, key=ontrackmap.get)]]

    def determine_ownership(self, item) -> list:
        """
        Determine opinionated ownership based on formal assignment via GitLab or highest note (comments) count.

        Args:
            item (enum): gitricsEpic || GitlabIssue

        Returns:
            A list of dictionaries where each is a GitlabUser.
        """

        result = None
        ids = list()
        subitem = getattr(item, item.gitlab_type)

        # check for notes
        if item.notes:

            # assign ownership by participation
            ids = [x[0] for x in Counter(d["author"]["id"] for d in item.notes).most_common(3)]

        # check for assignees
        if "assignees" in subitem and subitem["assignees"]:

            # add as assignees
            ids = [d["id"] for d in subitem["assignees"]]

        # replace ids with full user objects
        result = [d for d in self.users if d.id in ids] if self.users else list()

        return result

    def determine_status(self, item: dict) -> str:
        """
        Determine human-readable opinionated status based on start/end/due dates and state.

        Args:
            item (dictionary): key/value pairs representing a GitLab Epic or Issue

        Returns:
            A string representing a status related to start/end time.
        """

        result = "ongoing"
        now = datetime.datetime.now()

        # opened
        if item["state"] == "opened":

            # ending date
            if "end_date" in item or "due_date" in item:

                # normalize between data types
                edate = item["end_date"] if "end_date" in item else item["due_date"]

                # end/due date in the past
                if edate and datetime.datetime.strptime(edate, configuration.DATE_ISO_8601) < now:
                    result = "past due"

            # starting date
            if "start_date" in item or "due_date" in item:

                # normalize between data types
                sdate = item["start_date"] if "start_date" in item else item["due_date"]

                # start date in the future
                if sdate and datetime.datetime.strptime(sdate, configuration.DATE_ISO_8601) > now:
                    result = "upcoming"

        # closed
        else:
            result = "complete"

        return result

    def determine_top_blockers(self, graph: nx.DiGraph):
        """
        Determine which issues are blocking the most issues inside the group.

        Args:
            graph (DiGraph): networkx directional graph

        Returns:
            A list of gitricsIssue where each represents a GitLab Issue.
        """

        # calculate degree centrality
        degreecentralitymap = nx.degree_centrality(graph)

        # remove root node
        degreecentralitymap.pop(0)

        # get max value
        max_block_centrality = max(degreecentralitymap, key=degreecentralitymap.get)

        # update individual issues with score
        for i in self.issues.items:
            i.score_degree_centrality = degreecentralitymap[i.id] if i.id in degreecentralitymap else 0

        # get any nodes with max value
        # sort by nest level which assumes that nodes with an even score that the higher in the tree the more of a blocker it is
        return sorted([
            d for d in self.issues.items
            if d.score_degree_centrality > 0 and
            degreecentralitymap[d.id] == degreecentralitymap[max_block_centrality]
        ], key=lambda d: nx.descendants(graph, d.id), reverse=True)

    def initialize_epics(self, epics: list) -> list:
        """
        Initialize epics as gitrics objects.

        Args:
            epics (list): dictionaries of GitLab Epic

        Returns:
            A tuple (0, 1) where 0 is a list of gitricsEpic objects where each represents a GitLab Epic and 1 is a networkx directional graph object representing the parent/child relationships in the set of epics.
        """

        graph = None
        items = None

        # build tuples of parent nodes
        parents = [(d.epic["parent_id"], d.id) for d in epics if d.epic["parent_id"]]

        # build tuples of unlinked nodes
        # i.e. direct children of root
        unlinked = [(0, d.id) for d in epics if d.epic["parent_id"] is None]

        # combine into list of links
        links = parents + unlinked

        # generate graph
        if links: graph = nx.DiGraph(links)

        # format result for gitrics
        items = [gitricsEpic(epic=d.epic, group_graph=graph) for d in epics]

        return (items, graph)

    def initialize_issues(self, issues: list = None) -> list:
        """
        Initialize issues as gitrics objects.

        Args:
            issues (list): dictionaries of GitLab Issue

        Returns:
            A list of gitricsIssue objects where each represents a GitLab Issue.
        """

        graph = None
        items = None

        # build blocker tuples
        blockers = [
            x for y in [
                [
                    (d.id, i["id"])
                    for i in d.links
                    if i["link_type"] == "blocks"
                ]
                for d in issues if d.links
            ] for x in y
        ] if issues else list()

        # build tuples of blockers which are unblocked themselves
        # i.e. these nodes are direct children of the root
        unblocked_blockers = [
            (0, d.id) for d in issues
            if d.links and len([i for i in d.links if i["link_type"] == "is_blocked_by"]) == 0 and len([i for i in d.links if i["link_type"] == "blocks"]) > 0
        ] if issues else list()

        # build tuples for complete unlinked issues
        # i.e. nodes are direct children of the root
        unlinked = [(0, d.id) for d in issues if not d.links] if issues else list()

        # combine list
        links = blockers + unblocked_blockers + unlinked

        # generate graph
        if links: graph = nx.DiGraph(links)

        # format result for gitrics
        items = [gitricsIssue(issue=d.issue, group_graph=graph) for d in issues]

        return (items, graph)

    def prune(self, items: list, param: dict) -> list:
        """
        Prune data by provided parameter.

        Args:
            items (list): classes of gitricsEpic or GitlabIssue
            param (dict): key/value pairs where each key is an attribute to filter on and corresponding value is the desired query value

        Returns:
            A list of where each represents a gitricsEpic or GitlabIssue, depending on the input.
        """

        result = items

        # loop through param keys
        for key in param:

            # determine attribute type to filter against
            class_type = "user" if key == "ownership" else None

            # reduce list by value
            subset = [
                d for d in result
                if getattr(d, key) and param[key] in [
                    getattr(x, class_type)["id"]
                    for x in getattr(d, key)
                ]
            ]

            # update result
            result = subset

        return result



import os

from gitrics import configuration

def param_accent_color(parser):
    """
    Add accent color parameter.
    """

    parser.add_argument("--accent-color",
        type=str,
        help="Valid CSS color value (hex, rgb, core css color name, hsl, --var)."
    )

def param_access(parser):
    """
    Add access parameter.
    """

    parser.add_argument("--access",
        type=int,
        default=configuration.USER_ACCESS,
        help="Minimum user access when querying for projects (30 = developer, 40 = maintainer, 50 = owner)."
    )

def param_api(parser):
    """
    Add api parameter.
    """

    parser.add_argument("--api",
        type=str,
        default=configuration.GITLAB_API_VERSION,
        help="GitLab instance-specific API version in the form of a base URL."
    )

def param_date_end(parser):
    """
    Add date-end parameter.
    """

    parser.add_argument("--date-end",
        type=str,
        default=configuration.DATE_END,
        help="ISO 8601 date value."
    )

def param_date_start(parser):
    """
    Add date-start parameter.
    """

    parser.add_argument("--date-start",
        type=str,
        default=configuration.DATE_START,
        help="ISO 8601 date value."
    )

def param_filter_group_namespaces(parser):
    """
    Add filter-group-namespaces parameter.
    """

    parser.add_argument("--filter-group-namespaces",
        type=str,
        default=configuration.FILTER_GROUP_NAMESPACES,
        help="Comma-delimited Gitlab Group namespaces."
    )

def param_filter_project_ids(parser):
    """
    Add filter-project-ids parameter.
    """

    parser.add_argument("--filter-project-ids",
        type=str,
        default=configuration.FILTER_PROJECT_IDS,
        help="Comma-delimited Gitlab Project ids."
    )

def param_filter_project_names(parser):
    """
    Add filter-project-names parameter.
    """

    parser.add_argument("--filter-project-names",
        type=str,
        default=configuration.FILTER_PROJECT_NAMES,
        help="Comma-delimited Gitlab Project names."
    )

def param_filter_project_names_starts_with(parser):
    """
    Add filter-project-names-starts-with parameter.
    """

    parser.add_argument("--filter-project-names-startswith",
        type=str,
        default=configuration.FILTER_PROJECT_NAMES_STARTSWITH,
        help="Comma-delimited Gitlab Project name prefixes."
    )

def param_gitlab_name(parser):
    """
    Add gitlab-name parameter.
    """

    parser.add_argument("--gitlab-name",
        type=str,
        default=configuration.GITLAB_NAME,
        help="Label for GitLab instance."
    )

def param_gitlab_url(parser):
    """
    Add gitlab-url parameter.
    """

    parser.add_argument("--gitlab-url",
        type=str,
        default=configuration.GITLAB_URL,
        help="GitLab instance url."
    )

def param_gitrics_user(parser):
    """
    Add gitrics-user parameter.
    """

    parser.add_argument("--gitrics-user",
        type=str,
        default=configuration.FILEPATH_GITRICS_USER,
        help="gitrics-user metadata file path."
    )

def param_group(parser):
    """
    Add group parameter.
    """

    parser.add_argument("--group",
        type=str,
        help="GitLab Group id."
    )

def param_member_only(parser):
    """
    Add member-only parameter.
    """

    parser.add_argument("--member-only",
        action="store_true",
        default=configuration.MEMBERSHIP,
        help="Only query for projects user is a member of."
    )

def param_personal_only(parser):
    """
    Add personal-only parameter.
    """

    parser.add_argument("--personal-only",
        action="store_true",
        default=configuration.PERSONAL,
        help="Only query for personal projects (i.e. those in the user namepsace)."
    )

def param_return_data(parser):
    """
    Add return-data parameter.
    """

    parser.add_argument("--return-data",
        action="store_true",
        default=False,
        help="Whether or not to return user data object."
    )

def param_simple(parser):
    """
    Add simple parameter.
    """

    parser.add_argument("--simple",
        action="store_true",
        default=configuration.SIMPLE,
        help="Whether or not to query Gitlab for simplified data from endpoints. The same number of records are returned with smaller key/value footprints."
    )

def param_token(parser):
    """
    Add token parameter.
    """

    parser.add_argument("--token",
        type=str,
        default=configuration.GITLAB_TOKEN,
        help="GitLab Personal Access Token or Deploy Token to authenticate API requests."
    )

def param_user(parser):
    """
    Add user parameter.
    """

    parser.add_argument("--user",
        type=str,
        required=True,
        help="GitLab user id."
    )

def param_visibility(parser):
    """
    Add visibility parameter.
    """

    parser.add_argument("--visibility",
        type=str,
        default=configuration.VISIBILITY,
        help="What visibility level (internal, public, private) to query for from Gitlab."
    )



#!/usr/bin/env python3

import argparse
import json
import os

from importlib.metadata import files, metadata, version

from gitrics import configuration
from gitrics.bin.parameters import param_api, param_gitrics_user, param_token

from glapi.user.user import GitlabUser

def main():
    """
    Generate metadata for dependencies.
    """

    # parse arguments
    opt = parse_args()

    # get package info
    package_name = "gitrics"
    package_path = str(files(package_name)[0].locate()).split("/")
    package_path.pop()
    package_directory_path = "/".join([d for d in package_path])
    package_version = version(package_name)
    package_url = metadata(package_name)["Home-page"]

    # get authors
    author_usernames = list()
    gitrics_authors = list()
    gitrics_maintainers = list()

    # open authors file from package
    if os.path.exists(os.path.join(package_directory_path, "AUTHORS")):
        with open(os.path.join(package_directory_path, "AUTHORS"), "r") as f:
            author_usernames = [d.strip() for d in f.readlines()]

    # if api provided query for data
    if opt.api and opt.token:

        # query api for user data
        for username in author_usernames:

            # initialize user
            gu = GitlabUser(
                token=opt.token,
                username=username,
                version=opt.api
            )

            # add to list
            gitrics_authors.append({
                "id": gu.user["id"],
                "name": gu.user["name"],
                "web_url": gu.user["web_url"],
            })

        # get project id
        project_id = gu.connection.query("projects", params={ "search": "gitrics", "simple": "true" })["data"][0]["id"]

        # format data
        gitrics_maintainers = [
            {
                "id": d["id"],
                "name": d["name"],
                "web_url": d["web_url"]
            }
            for d in gu.connection.query("projects/%s/members" % project_id)["data"]
            if d["access_level"] >= 40
        ]

    # open gitrics user file
    gitrics_user = dict()
    if opt.gitrics_user and os.path.exists(opt.gitrics_user):
        with open(opt.gitrics_user, "r") as f:
            gitrics_user = json.loads(f.read())

    # write metadata to file
    with open("dependencies.json", "w") as f:
        f.write(json.dumps({
            "gitrics": {
                "authors": gitrics_authors,
                "maintainers": gitrics_maintainers,
                "repo_url": package_url,
                "version_label": package_version,
                "version_url": "%s/-/tags/%s" % (
                    package_url,
                    package_version
                )
            },
            "gitrics_user": {
                "authors": gitrics_user["authors"] if gitrics_user else None,
                "maintainers": gitrics_user["maintainers"] if gitrics_user else None,
                "repo_url": gitrics_user["repo_url"] if gitrics_user else None,
                "version_label": gitrics_user["version_label"] if gitrics_user else None,
                "version_url": gitrics_user["version_url"] if gitrics_user else None
            }
        }))

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define optional values
    param_api(parser)
    param_gitrics_user(parser)
    param_token(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



import networkx as nx

from glapi import configuration
from glapi.issue import GitlabIssue

class gitricsIssue(GitlabIssue):
    """
    gitricsIssue is a Gitlab Issue with opinionated enrichment for the gitrics ecosystem.
    """

    def __init__(self, id: str = None, iid: str = None, issue: dict = None, group_id: str = None, group_graph: nx.DiGraph = None, token: str = configuration.GITLAB_TOKEN, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            id (string): GitLab Issue id
            iid (string): GitLab Issue iid
            group_id (string): Gitlab Group id
            group_graph (DiGraph): networkx directional graph
            issue (dictionary): GitLab Issue
            token (string): GitLab personal access, ci, or deploy token
            version (string): GitLab API version as base url
        """
        self.gitlab_type = "issue"
        self.group_graph = group_graph
        self.ownership = None

        # initialize inheritance
        super(gitricsIssue, self).__init__(
            issue=issue,
            token=token,
            version=version
        )



from gitrics import configuration

from glapi.user import GitlabUser

class gitricsUserIssueMergeRequest(GitlabUser):
    """
    gitricsUserIssueMergeRequest is an abstraction of the GitLab Issue and GitLab Merge Request data objects specific to the requirements of the gitrics ecosystem.
    """

    def __init__(self, type: str, user_id: str = None, user: dict = None, date_start: str = None, date_end: str = None, token: str = None, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            date_end (string): iso 8601 date value
            date_start (string): iso 8601 date value
            token (string): GitLab personal access or deploy token
            type (enum): issue | merge_request
            user (dict): key/values representing a Gitlab User
            user_id (string): GitLab User id
            version (string): GitLab API version as base url
        """

        self.assigned_to_user = None
        self.created_by_user = None
        self.items = None

        # initialize inheritance
        super(gitricsUserIssueMergeRequest, self).__init__(
            id=user_id,
            token=token,
            user=user,
            version=version
        )

        # get items
        self.created_by_user = self.extract_merge_requests(
            date_end=date_end,
            date_start=date_start,
            scope="created_by_me"
        ) if type == "merge_request" else self.extract_issues(
            date_end=date_end,
            date_start=date_start,
            scope="created_by_me"
        )
        self.created_by_user = self.created_by_user if self.created_by_user else list()

        # get items
        self.assigned_to_user = self.extract_merge_requests(
            date_end=date_end,
            date_start=date_start,
            scope="assigned_to_me"
        ) if type == "merge_request" else self.extract_issues(
            date_end=date_end,
            date_start=date_start,
            scope="assigned_to_me"
        )
        self.assigned_to_user = self.assigned_to_user if self.assigned_to_user else list()

        # update self
        self.items = self.created_by_user + self.assigned_to_user

    def format(self, items: list) -> dict:
        """
        Generate key/value map for user id to correlated user objects to represent connected GitLab Users.

        Args:
            items (list): dictionaries where each is a GitLab MergeRequest or Issue

        Returns:
            A dictionary where each key is a user id representing an issue author or assignee the core user has interaction with and the corresponding values are the count of connections and the GitLab User object for the connected user id.
        """

        result = dict()

        # check for items
        if items:

            # loop through GitLab objects
            for item in items:

                # pull current ids
                assignee_id = item["assignee"]["id"] if item["assignee"] else None
                author_id = item["author"]["id"] if item["author"] else None

                # if there is an assignee and it's not the core user
                if assignee_id and hasattr(self, "user") and assignee_id != self.user["id"]:

                    # check for existing key
                    if assignee_id not in result:

                        # add key
                        result[assignee_id] = {
                            "count": 0,
                            "user": item["assignee"]
                        }

                    # iterate count
                    result[assignee_id]["count"] += 1

                # if there is an author and it's not the core user
                if author_id and hasattr(self, "user") and author_id != self.user["id"]:

                    # check for existing key
                    if author_id not in result:

                        # add key
                        result[author_id] = {
                            "count": 0,
                            "user": item["author"]
                        }

                    # iterate count
                    result[author_id]["count"] += 1

        return result

    def prune(self, items: list) -> list:
        """
        Prune items which are both created by and assigned to user.

        Args:
            items (list): dictionaries where each is a Gitlab MergeRequest or Issue

        Returns:
            A list of dictionaries where each represents a Gitlab MergeRequest or Issue.
        """

        result = list()

        # check for items
        if items:

            # filter out created by user and assigned to user
            created_filtered = [
                d for d in self.items
                if d["assignee"] and hasattr(self, "user") and d["assignee"]["id"] != self.user["id"]
              ]

            # filter out created by user and assigned to user
            assigned_filtered = [
                d for d in self.items
                if d["author"] and hasattr(self, "user") and d["author"]["id"] != self.user["id"]
            ]

            # update result
            result = created_filtered + assigned_filtered

        return result



from gitrics import configuration
from gitrics.user.issue_merge_request import gitricsUserIssueMergeRequest

class gitricsUserIssues(gitricsUserIssueMergeRequest):
    """
    gitricsUserIssues is a collection of user-specific Gitlab Issue objects modified and enriched for gitrics ecosystem.
    """

    def __init__(self, user_id: str = None, user: dict = None, date_start: str = None, date_end: str = None, token: str = None, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            date_end (string): iso 8601 date value
            date_start (string): iso 8601 date value
            token (string): GitLab personal access or deploy token
            user (dict): key/values representing a Gitlab User
            user_id (string): GitLab User id
            version (string): GitLab API version as base url
        """

        # initialize inheritance
        super(gitricsUserIssues, self).__init__(
            date_end=date_end,
            date_start=date_start,
            token=token,
            type="issue",
            user=user,
            user_id=user_id,
            version=version
        )



#!/usr/bin/env python3

import argparse
import json
import math
import os

from collections import Counter

from gitrics import configuration
from gitrics.bin.parameters import param_api, param_date_end, param_date_start, param_token, param_user
from gitrics.user.events import gitricsUserEvents
from gitrics.utilities import configure_date_range

def main(user: dict = None, date_start: str = None, date_end: str = None, projects: list = None):
    """
    Generate activity for user from GitLab usage.

    Args:
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value
        projects (list): dictionaries where each represents a GitLab Project
        user (dictionary): GitLab User
    """

    # parse arguments
    opt = parse_args()

    # convert time
    dt_start, dt_end = configure_date_range(
        date_end=date_end if date_end else opt.date_end,
        date_start=date_start if date_start else opt.date_start
    )

    # convert time
    count_days = (dt_end - dt_start).days

    # initialize events
    user_events = gitricsUserEvents(
        date_end=dt_end.strftime("%Y-%m-%d"),
        date_start=dt_start.strftime("%Y-%m-%d"),
        token=opt.token,
        user_id=opt.user,
        version=opt.api
    )

    # flatten all events into single list
    events = [item for sublist in [user_events.events[k] for k in user_events.events] for item in sublist]

    events_coding = user_events.events["pushed"] + user_events.events["merged"]

    events_collaboration = user_events.events["approved"] + user_events.events["closed"] + user_events.events["commented"] + user_events.events["created"] + user_events.events["updated"]

    # totals
    total_events_coding = len(events_coding)
    total_events_collaboration = len(events_collaboration)
    total_projects_with_events = len(list(set([d["project_id"] for d in events])))

    # get common factor
    event_type_common_factor = math.gcd(total_events_coding, total_events_collaboration) if math.gcd(total_events_coding, total_events_collaboration) > 0 else 1

    # get activity stats
    count_activity_types = count_activity_types = { k: sum([len(user_events.events[a]) for a in configuration.ACTIVITY_TYPES[k]]) for k in configuration.ACTIVITY_TYPES }

    # count date occurances in events
    date_occurances_in_events = [d["created_at"].split("T")[0] for d in events]
    date_occurance_count = Counter(date_occurances_in_events)
    
    # write to file
    with open("activity.json", "w") as f:
        f.write(json.dumps({
            "stats": {
                "average_coding_per_day": int(round(count_activity_types["coding"] / count_days)),
                "average_coding_per_project": int(round(count_activity_types["coding"] / total_projects_with_events)) if total_projects_with_events > 0 else 0,
                "average_collaboration_per_day": int(round(count_activity_types["collaboration"] / count_days)),
                "average_collaboration_per_project": int(round(count_activity_types["collaboration"] / total_projects_with_events)) if total_projects_with_events > 0 else 0,
                "max_activity_date_count": date_occurance_count.most_common(1)[0][1] if len(date_occurance_count) > 0 else None,
                "max_activity_date": date_occurance_count.most_common(1)[0][0] if len(date_occurance_count) > 0 else None,
                "rate_collaboration_per_code": "%s:%s" % (
                    int(round(total_events_collaboration / event_type_common_factor)),
                    int(round(total_events_coding / event_type_common_factor))
                )
            },
            "data": {
                "coding": [
                    {
                        "date": d["created_at"],
                        "value": 1,
                        "type": "coding"
                    } for d in events_coding
                ],
                "collaboration": [
                    {
                        "date": d["created_at"],
                        "value": 1,
                        "type": "collaboration"
                    } for d in events_collaboration
                ]
            }
        }))

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_api(parser)
    param_date_end(parser)
    param_date_start(parser)
    param_token(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import json
import os

from gitrics import configuration
from gitrics.bin.generate_user_project_membership import main as ProjectMembership
from gitrics.bin.parameters import param_access, param_api, param_date_end, param_date_start, param_member_only, param_personal_only, param_simple, param_token, param_user, param_visibility
from gitrics.user.projects import gitricsUserProjects
from gitrics.utilities import configure_date_range

from glapi.user.user import GitlabUser

def main(user: dict = None, date_start: str = None, date_end: str = None, projects: list = None, projects_binned_by_access: dict = None, count_projects_private: int = None):
    """
    Generate user data coverage from GitLab usage.

    Args:
        count_projects_private (integer): number of private projects
        date_end (string): iso 8601 date value
        date_start (string): iso 8601 date value
        projects (list): dictionaries where each represents a GitLab Project
        projects_binned_by_access (dict): key/value pairs of access to GitLab Projects
        user (str): GitLab user id

    """

    # parse arguments
    opt = parse_args()

    # convert time
    dt_start, dt_end = configure_date_range(
        date_end=date_end if date_end else opt.date_end,
        date_start=date_start if date_start else opt.date_start
    )

    # initialize user if necessary
    if not user:
        user = GitlabUser(
            id=opt.user,
            token=opt.token,
            version=opt.api
        ).user

    # get user project membership if necessary
    if not projects:
        projects, projects_binned_by_access, count_projects_private = ProjectMembership(
            date_end=dt_end.strftime("%Y-%m-%d"),
            date_start=dt_start.strftime("%Y-%m-%d"),
            return_data=True,
            user=user
        )

    # calculate totals
    total_projects = len(projects)

    total_projects_personal = len([d for d in projects_binned_by_access["maintainer"] if user["username"] in d["path_with_namespace"]]) if projects_binned_by_access and user else 0
    total_projects_public = total_projects - total_projects_personal

    # write user to file
    with open("coverage.json", "w") as f:
        f.write(json.dumps({
            "stats": {
                "percent_projects_coverage": int(round(total_projects_public / (total_projects + count_projects_private) * 100))
            }
        }))

def parse_args():
    """
    Parse user input.

    Returns:
        A Namespace with attributes for each provided argument.
    """

    # create arguments
    parser = argparse.ArgumentParser()

    # define required values
    param_user(parser)

    # define optional values
    param_access(parser)
    param_api(parser)
    param_date_end(parser)
    param_date_start(parser)
    param_member_only(parser)
    param_personal_only(parser)
    param_simple(parser)
    param_token(parser)
    param_visibility(parser)

    # parse user input
    args, unknown = parser.parse_known_args()

    return args

if __name__ == "__main__":
    main()



import os

from setuptools import setup, find_packages

setup(
    name="gitrics",
    description="Python package to generate metrics based off git usage.",
    long_description=open(os.path.join(os.getcwd(), "README.md")).read().strip(),
    long_description_content_type="text/markdown",
    version=open(os.path.join(os.getcwd(), "VERSION")).read().strip(),
    url="https://gitlab.com/lgensinger/gitrics",
    install_requires=[d.strip() for d in open(os.path.join(os.getcwd(), "requirements.txt")).readlines()],
    extras_require={
        "test": [d.strip() for d in open(os.path.join(os.getcwd(), "requirements-test.txt")).readlines()]
    },
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "generate-dependency-metadata=gitrics.bin.generate_dependency_metadata:main",
            "generate-group-blockers=gitrics.bin.generate_group_blockers:main",
            "generate-metadata=gitrics.bin.generate_metadata:main",
            "generate-user-activity=gitrics.bin.generate_user_activity:main",
            "generate-user-color=gitrics.bin.generate_user_color:main",
            "generate-user-connection=gitrics.bin.generate_user_connection:main",
            "generate-user-contribution=gitrics.bin.generate_user_contribution:main",
            "generate-user-data=gitrics.bin.generate_user_data:main",
            "generate-user-data-coverage=gitrics.bin.generate_user_data_coverage:main",
            "generate-user-languages=gitrics.bin.generate_user_languages:main",
            "generate-user-profile=gitrics.bin.generate_user_profile:main",
            "generate-user-project-membership=gitrics.bin.generate_user_project_membership:main",
            "generate-user-topics=gitrics.bin.generate_user_topics:main"
        ],
    }
)



import networkx as nx

from glapi import configuration
from glapi.epic import GitlabEpic

class gitricsEpic(GitlabEpic):
    """
    gitricsEpic is a Gitlab Epic with opinionated enrichment for the gitrics ecosystem.
    """

    def __init__(self, id: str = None, iid: str = None, epic: dict = None, group_id: str = None, group_graph: nx.DiGraph = None, token: str = configuration.GITLAB_TOKEN, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            id (string): GitLab Epic id
            iid (string): GitLab Epic iid
            epic (dictionary): GitLab Epic
            group_graph (DiGraph): networkx directional graph
            group_id (string): Gitlab Group id
            token (string): GitLab personal access, ci, or deploy token
            version (string): GitLab API version as base url
        """
        self.gitlab_type = "epic"
        self.group_graph = group_graph
        self.issues = None
        self.count_at_risk = 0
        self.count_on_track = 0
        self.ownership = None
        self.percent_complete = 0
        self.score_at_risk = 0
        self.score_on_track = 0

        # initialize inheritance
        super(gitricsEpic, self).__init__(
            epic=epic,
            group_id=group_id,
            iid=iid,
            token=token,
            version=version
        )

    def calculate_completion(self, epics: list, issues: list = None) -> int:
        """
        Calculate completion based on closed/opened issues and epics.

        Args:
            epics (list): gitricsEpic classes where each represents a GitLab Epic
            list (list): gitricsIssue classes where each represents a GitLab Issue

        Returns:
            An integer of a percent representation as a whole number.
        """

        result = 0

        # calculate subepics
        epics_total = len(epics)
        epics_closed = len([d for d in epics if d.epic["state"] == "closed"])

        # calculate issues
        issues = issues if issues else self.extract_issues()

        # update self since we have to pull issues anyway
        self.issues = issues

        issues_total = len(issues) if issues else 0
        issues_closed = len([d for d in issues if d.issue["state"] == "closed"]) if issues else 0

        # combine epic/issue totals
        total = epics_total + issues_total
        total_closed = epics_closed + issues_closed

        # get percent as decimal
        dec = total_closed / total if total != 0 else None

        # round to nearest int
        if dec: result = int(round(dec * 100))

        return result



from gitrics import configuration

from glapi.group import GitlabGroup
from glapi.user import GitlabUser

class gitricsGroupUsers(GitlabGroup):
    """
    gitricsGroupUsers is a collection of Gitlab Group Users modified and enriched for gitrics ecosystem.
    """

    def __init__(self, group_id: str = None, group: dict = None, users: list = None, token: str = configuration.GITLAB_TOKEN, version: str = configuration.GITLAB_API_VERSION):
        """
        Args:
            group (dictionary): key/value pair representing a GitLab Group
            group_id (string): GitLab Group id
            token (string): GitLab personal access, ci, or deploy token
            users (list): dictionaries of GitLab User
            version (string): GitLab API version as base url
        """
        self.gitrics_type = "users"

        # initialize inheritance
        super(gitricsGroupUsers, self).__init__(
            group=group,
            id=group_id,
            token=token,
            version=version
        )

        # get users
        if users:
            self.users = [GitlabUser(user=d) for d in users]
        else:
            # try to get from api
            data = self.extract_users()

            # format result
            self.users = [GitlabUser(user=d.user) for d in data] if data else None
